@@ -19,9 +19,9 @@
 
 """Provides the FocusTrackingPresenter for Orca."""
 
-__id__        = "$Id: focus_tracking_presenter.py 2551 2007-07-24 17:13:10Z richb $"
-__version__   = "$Revision: 2551 $"
-__date__      = "$Date: 2007-07-24 13:13:10 -0400 (Tue, 24 Jul 2007) $"
+__id__        = "$Id: focus_tracking_presenter.py 2573 2007-07-31 16:11:39Z richb $"
+__version__   = "$Revision: 2573 $"
+__date__      = "$Date: 2007-07-31 12:11:39 -0400 (Tue, 31 Jul 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -413,6 +413,17 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                          self._knownAppSettings[name].overrideAppKeyBindings( \
                             script, script.keyBindings)
 
+                    # Setup the user's application specific pronunciations
+                    # (if any).
+                    #
+                    if hasattr(self._knownAppSettings[name],
+                               "overridePronunciations"):
+                        script.overridePronunciations = \
+                            self._knownAppSettings[name].overridePronunciations
+                        script.app_pronunciation_dict = \
+                         self._knownAppSettings[name].overridePronunciations( \
+                            script, script.app_pronunciation_dict)
+
                     break
                 except ImportError:
                     debug.println(debug.LEVEL_FINEST,
