@@ -21,9 +21,9 @@
 fallback to local settings if the user settings doesn't exist (e.g., in the
 case of gdm) or doesn't have the specified attribute."""
 
-__id__        = "$Id: settings.py 2521 2007-07-06 20:46:49Z mpeders $"
-__version__   = "$Revision: 2521 $"
-__date__      = "$Date: 2007-07-06 16:46:49 -0400 (Fri, 06 Jul 2007) $"
+__id__        = "$Id: settings.py 2581 2007-08-01 15:29:21Z richb $"
+__version__   = "$Revision: 2581 $"
+__date__      = "$Date: 2007-08-01 11:29:21 -0400 (Wed, 01 Aug 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -592,6 +592,11 @@ def setGKSUGrabDisabled(disable):
 def overrideKeyBindings(script, keyBindings):
     return keyBindings
 
+# Allow for user customization of pronunciations.
+#
+def overridePronunciations(script, pronunciations):
+    return pronunciations
+
 # Which packages to search, and the order in which to search,
 # for application settings.  These packages are expected to be on
 # the PYTHONPATH and/or subpackages of the "orca" package.
@@ -671,6 +676,11 @@ setScriptMapping(re.compile(_('[\S\s]*StarOffice[\s\S]*')), "StarOffice")
 #
 setScriptMapping(re.compile(_('soffice.bin')), "StarOffice")
 
+# Translators: see the regular expression note above.  This is for
+# OpenOffice and StarOffice.
+#
+setScriptMapping(re.compile(_('soffice')), "StarOffice")
+
 # Translators: see the regular expression note above.  This is for the
 # Evolution mail application.
 #
