@@ -23,9 +23,9 @@ class.  For those wishing to override the speech generators, however,
 one can create a new instance and replace/extend the speech generators
 as they see fit."""
 
-__id__        = "$Id: speechgenerator.py 2280 2007-04-11 15:54:35Z wwalker $"
-__version__   = "$Revision: 2280 $"
-__date__      = "$Date: 2007-04-11 11:54:35 -0400 (Wed, 11 Apr 2007) $"
+__id__        = "$Id: speechgenerator.py 2719 2007-08-26 18:52:54Z joanied $"
+__version__   = "$Revision: 2719 $"
+__date__      = "$Date: 2007-08-26 14:52:54 -0400 (Sun, 26 Aug 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -37,7 +37,8 @@ import orca_state
 import rolenames
 import settings
 
-from orca_i18n import _                          # for gettext support
+from orca_i18n import _   # for gettext support
+from orca_i18n import Q_  # to provide qualified translatable strings
 
 class SpeechGenerator:
     """Takes accessible objects and produces a string to speak for
@@ -795,6 +796,18 @@ class SpeechGenerator:
             utterances.extend(self._getSpeechForObjectAvailability(obj))
             utterances.extend(self._getSpeechForObjectAccelerator(obj))
 
+        # OpenOffice check menu items currently have a role of "menu item"
+        # rather then "check menu item", so we need to test if one of the
+        # states is CHECKED. If it is, then add that in to the list of
+        # speech utterances. Note that we can't tell if this is a "check
+        # menu item" that is currently unchecked and speak that state. 
+        # See Orca bug #433398 for more details.
+        #
+        if obj.state.count(atspi.Accessibility.STATE_CHECKED):
+            # Translators: this represents the state of a checked menu item.
+            #
+            utterances.append(_("checked"))
+
         self._debugGenerator("_getSpeechForMenuItem",
                              obj,
                              already_focused,
@@ -907,9 +920,12 @@ class SpeechGenerator:
         Returns a list of utterances to be spoken for the object.
         """
 
+        percentValue = (obj.value.currentValue / \
+            (obj.value.maximumValue - obj.value.minimumValue)) * 100.0
+
         # Translators: this is the percentage value of a progress bar.
         #
-        percentage = _("%d percent.") % obj.value.currentValue + " "
+        percentage = _("%d percent.") % percentValue + " "
 
         utterances = []
 
@@ -965,12 +981,16 @@ class SpeechGenerator:
             # Translators: this is in reference to a radio button being
             # selected or not.
             #
-            selectionState = _("selected")
+            # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
+            #
+            selectionState = Q_("radiobutton|selected")
         else:
             # Translators: this is in reference to a radio button being
             # selected or not.
             #
-            selectionState = _("not selected")
+            # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
+            #
+            selectionState = Q_("radiobutton|not selected")
 
         # If it's not already focused, say it's name
         #
@@ -1221,6 +1241,56 @@ class SpeechGenerator:
 
         utterances = []
 
+        # Check to see if this table cell contains an icon (image).
+        # If yes:
+        #   1/ Try to get a description for it and speak that. 
+        #   2/ Treat the object of role type ROLE_IMAGE and speak 
+        #      the role name.
+        # See bug #465989 for more details.
+        #
+        if obj.image:
+            if obj.image.imageDescription:
+                utterances.append(obj.image.imageDescription)
+            obj.role = rolenames.ROLE_IMAGE
+            utterances.extend(self._getSpeechForImage(obj, already_focused))
+            obj.role = rolenames.ROLE_TABLE_CELL
+            return utterances
+
+        # If this table cell has 2 children and one of them has a 
+        # 'toggle' action and the other does not, then present this 
+        # as a checkbox where:
+        # 1) we get the checked state from the cell with the 'toggle' action
+        # 2) we get the label from the other cell.
+        # See Orca bug #376015 for more details.
+        #
+        if obj.childCount == 2:
+            cellOrder = []
+            hasToggle = [ False, False ]
+            for i in range(0, obj.childCount):
+                action = obj.child(i).action
+                if action:
+                    for j in range(0, action.nActions):
+                        if action.getName(j) == "toggle":
+                            hasToggle[i] = True
+                            break
+
+            if hasToggle[0] and not hasToggle[1]:
+                cellOrder = [ 1, 0 ] 
+            elif not hasToggle[0] and hasToggle[1]:
+                cellOrder = [ 0, 1 ]
+            if cellOrder:
+                for i in cellOrder:
+                    # Don't speak the label if just the checkbox state has
+                    # changed.
+                    #
+                    if already_focused and not hasToggle[i]:
+                        pass
+                    else:
+                        utterances.extend( \
+                            self._getSpeechForTableCell(obj.child(i),
+                                                              already_focused))
+                return utterances
+
         # [[[TODO: WDW - Attempt to infer the cell type.  There's a
         # bunch of stuff we can do here, such as check the EXPANDABLE
         # state, check the NODE_CHILD_OF relation, etc.  Logged as
@@ -1314,6 +1384,21 @@ class SpeechGenerator:
                         showing = cell.state.count( \
                                           atspi.Accessibility.STATE_SHOWING)
                         if showing:
+                            # If this table cell has a "toggle" action, and
+                            # doesn't have any label associated with it then 
+                            # also speak the table column header.
+                            # See Orca bug #455230 for more details.
+                            #
+                            label = self._script.getDisplayedText( \
+                                self._script.getRealActiveDescendant(cell))
+                            action = cell.action
+                            if action and (label == None or len(label) == 0):
+                                for j in range(0, action.nActions):
+                                    if action.getName(j) == "toggle":
+                                        header = parent.table.getColumnHeader(i)
+                                        accHeader = atspi.Accessible.makeAccessible(header)
+                                        utterances.append(accHeader.name)
+
                             utterances.extend(self._getSpeechForTableCell(cell,
                                                            already_focused))
                 else:
